////////////////////////////////////////////////////////////////////////////////
/*! \file       auto_xerces_ptr.hpp
    Header file for the auto_xerces_ptr class

    Version:    1.0                       <br>
    Copyright:  Orjan Westin 2010         <br>
    Licence:    BSD (see EOF comment)     <br>
    Contact:    orjan@coolcowstudio.co.uk <br>
    Download:   http://coolcowstudio.co.uk/source/cpp/xerces_ptr.zip
*///////////////////////////////////////////////////////////////////////////////

#ifndef COCOST_AUTO_XERCES_PTR_H_
#define COCOST_AUTO_XERCES_PTR_H_

#include <xercesc/dom/DOM.hpp>

//! Cool Cow Studio namespace
namespace CoCoSt 
{

//! RIAA adapter for dynamic Xerces types 
/*! Wraps DOMNode*, XMLCh* and char* and uses the Xerces release mechanism to 
    automatically free their memory when going out of scope.
    The Xerces-C++ XML parser can be downloaded from:
    http://xerces.apache.org/xerces-c/
*/
template <typename T>
class auto_xerces_ptr
{
    // Hide copy constructor and assignment operator
    auto_xerces_ptr(const auto_xerces_ptr&);
    auto_xerces_ptr& operator=(const auto_xerces_ptr&);

    // Release Xerces data type with a release member function
    template <typename T> 
    static void do_release(T* item)
    {
        // Only release this if it has no owner
        if (0 == item->getOwnerDocument())
            item->release();
    }
    // Specializations for character types, released by XMLString::release
    template <> 
    static void do_release(char* item)
    {
        XMLString::release(&item);
    }
    template <> 
    static void do_release(XMLCh* item)
    {
        XMLString::release(&item);
    }
  
    // The actual data
    T* item_;
public:
    //! Default constructor
    auto_xerces_ptr()
        : item_(0)
    {}
    //! Assignment constructor
    /*! \param item pointer to data to take ownership of
    */
    explicit auto_xerces_ptr(T* item)
        : item_(item)
    {}
    //! Destructor
    ~auto_xerces_ptr()
    {
        xerces_release();
    }
    //! Assignment of data to guard (not chainable)
    /*! \param item pointer to data to take ownership of
    */
    void operator=(T* item)
    {
        assign(item);
    }
    //! Delete held data, if any 
    void xerces_release()
    {
        if (!is_released())
        {
            // Use type-specific release mechanism
            do_release(item_);
            item_ = 0;
        }
    }
    //! Give up held data without deleting
    /*! \return pointer to data, if any
    */
    T* yield()
    {
        T* tempItem = item_;
        item_ = 0;
        return tempItem;
    }
    //! Assignment of data (releases previously held data, if any)
    /*! \param item pointer to data to take ownership of
    */
    void assign(T* item)
    {
        xerces_release();
        item_ = item;
    }
    //! Get pointer to the currently held data
    /*! \return pointer to data, if any
    */
    T* get()
    {
        return item_;
    }
    //! Get pointer to the currently held data
    /*! \return pointer to data, if any
    */
    const T* get() const
    {
        return item_;
    }
    //! Check if any data is held
    /*! \return true if no data is held
    */
    bool is_released() const
    {
        return (0 == item_);
    }
};

}  // namespace CoCoSt

#endif // COCOST_AUTO_XERCES_PTR_H_

/*

Copyright (c) 2010, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may not be used to endorse or promote products 
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/